#include "MX5Theatre.h"

MX5Theatre::MX5Theatre(QWidget *parent)
    : QWidget(parent)
{
	// QT GUI SETUP
	ui.setupUi(this);
	QWidget::showFullScreen();

	qdw = QApplication::desktop();
	QRect screen_dim = qdw->availableGeometry(0);

	int space = 10; // Number of pixels btwn border and elements
	int pbw = ui.playPauseButton->width();
	int pbh = ui.playPauseButton->height();

	ui.playPauseButton->move(screen_dim.right() - (pbw*2 + space*6),
								screen_dim.bottom() - (pbh + space*4));

	ui.stopButton->move(screen_dim.right() - (pbw + space*4),
								screen_dim.bottom() - (pbh + space*4));

	ui.fullButton->setGeometry(screen_dim.left() - 10,		// These extra 10s and 20s
								screen_dim.top() - 10,		// hide button's dotted-edge
								screen_dim.width() + 20,	// off-screen
								screen_dim.height() + 20);

	ui.instructionLabel->move((screen_dim.width() - ui.instructionLabel->width())/2,
								(screen_dim.height() - ui.instructionLabel->height())/2);

	ui.usageLabel->move(space*6,
							screen_dim.height() - (ui.usageLabel->height()) - space*4);
	ui.usageLabel->hide();

	// Setting up and hiding splash image
	splash_label = new QLabel(this);
	splash_pixmap = QPixmap(QString::fromUtf8(":/timesys.jpg"));
	splash_label->setPixmap(splash_pixmap);
	splash_label->resize(splash_pixmap.width(), splash_pixmap.height());

	splash_label->move((screen_dim.width() - splash_label->width())/2,
						(screen_dim.height() - splash_label->height())/2);
	splash_label->hide();
	splash_palette.setBrush(this->backgroundRole(), QBrush(QColor(0, 0, 0)));
	this->setPalette(splash_palette);

	// Center cursor
	QCursor::setPos(screen_dim.left() + screen_dim.width()/2,
					screen_dim.top() + screen_dim.height()/2);

	// Initialize timer with 1s callback
	timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(callback()));
	timer->start(1000);
	first_timeout = true;

	// GSTREAMER INITIALIZATION
	gst_player = new GstPlayer();
	gst_bus = gst_player->get_bus();
	set_alpha(255);	// Overrides gstreamer alpha settings

	// Default: is not playing
	is_playing = false;
	is_stopped = true;
}

MX5Theatre::~MX5Theatre()
{
}

void
MX5Theatre::clicked()
{
	if(is_playing)
	{
		set_alpha(127);
		ui.playPauseButton->show();
		ui.stopButton->show();
		ui.usageLabel->show();
	}
}

void
MX5Theatre::play_pause()
{
	// Play stream
	if(!is_playing){

		gst_player->play();
		ui.playPauseButton->setIcon(QIcon(":/pause.jpg"));

		paint_splash(false);
		ui.instructionLabel->hide();
		ui.playPauseButton->hide();
		ui.stopButton->hide();
		ui.usageLabel->hide();

		// If beginning to play the first time, we have to pause to let
		// GStreamer get its ducks in a row
		if(is_stopped)
		{
			usleep(1000000);
			is_stopped = false;
		}
		set_alpha(0);
	}
	// Pause stream
	else{
		gst_player->pause();
		ui.playPauseButton->setIcon(QIcon(":/play.jpg"));
	}

	is_playing = !is_playing; // Invert value of is_playing
}

// Reset pipeline to beginning of stream
void
MX5Theatre::stop()
{
	reset_player();
	is_playing = false;
	is_stopped = true;
	ui.playPauseButton->setIcon(QIcon(":/play.jpg"));
	ui.playPauseButton->show();
	ui.stopButton->show();
	ui.usageLabel->hide();
	paint_splash(true);

	usleep(500000);
	set_alpha(255);
}

// Dereference pipeline, end program
void
MX5Theatre::end()
{
	gst_player->destroy();

	delete(timer);
	delete(gst_player);

	exit(0);
}

void
MX5Theatre::reset_player()
{
	gst_player->stop();
	gst_bus = gst_player->get_bus();
}

void
MX5Theatre::callback()
{
	show_usage();
	// If EOS reached, call bus
	if(check_bus())
	{
		stop();
	}
}


// CPU usage (%) = (CPU time utilized / real time passed) * 100
void
MX5Theatre::show_usage(){

	gettimeofday(&rt_current,NULL);
	rt_current_d = (double)rt_current.tv_sec + (double)rt_current.tv_usec/1000000.0;

	getrusage(RUSAGE_SELF,&ct_current); // if process discrepancy, change constant here
	ct_user_current = ct_current.ru_utime;
	ct_sys_current = ct_current.ru_stime;

	ct_current_d = (double)ct_user_current.tv_sec + (double)ct_user_current.tv_usec/1000000.0
					+ (double)ct_sys_current.tv_sec + (double)ct_sys_current.tv_usec/1000000.0;

	// Run this if second+ time through showUsage
	if(!first_timeout)
	{
		rt_passed_d = rt_current_d - rt_last_d;
		ct_passed_d = ct_current_d - ct_last_d;

		cpu_usage = (ct_passed_d / rt_passed_d) * 100.0;

		sprintf(myString,"CPU Usage: %.f%%", cpu_usage);
		ui.usageLabel->setText(QString(myString));
	}
	else first_timeout = false;

	rt_last_d = rt_current_d;
	ct_last_d = ct_current_d;
}

// Pass true if you want to fill with TimeSys logo/
// Otherwise, pass false for black
void
MX5Theatre::paint_splash(bool fill)
{
	if(fill == true)
	{
		splash_palette.setBrush(this->backgroundRole(), QBrush(QColor(0, 84, 128)));
		this->setPalette(splash_palette);
		splash_label->show();
	}
	else
	{
		splash_palette.setBrush(this->backgroundRole(), QBrush(QColor(0, 0, 0)));
		this->setPalette(splash_palette);
		splash_label->hide();
	}
}

// Return true if EOS; else, returns false
bool
MX5Theatre::check_bus(){

	GstMessage *msg;
	bool eos_reached = false;

	while((msg = gst_bus_pop(gst_bus)) != NULL){
		// g_print("\nBus got message!");
		switch(GST_MESSAGE_TYPE(msg)){
			case GST_MESSAGE_EOS: {
				g_print("\nEOS reached. Now stopping pipeline");
				eos_reached = true;
				break;
			}
			case GST_MESSAGE_ERROR: {
				g_print("\nGstreamer error");
				GError *error;
				gchar *debug;
				gst_message_parse_error(msg, &error, &debug);
				g_print("\nError: %s", error->message);
				g_error(debug);
				g_error_free(error);
				break;
			}
			default:
				break;
		}
		gst_message_unref(msg);
	}
	return eos_reached;
}
